@@ -40,7 +40,7 @@
 #include "gpm-debug.h"
 
 #include "gpm-manager.h"
-#include "gpm-interface-manager.h"
+#include "dbus/gpm-dbus-manager.h"
 
 static void gpm_exit (GpmManager *manager);
 
@@ -61,28 +61,37 @@ gpm_object_register (DBusGConnection *connection,
 	DBusGProxy *bus_proxy = NULL;
 	GError *error = NULL;
 	guint request_name_result;
+	gboolean ret;
 
 	bus_proxy = dbus_g_proxy_new_for_name (connection,
 					       DBUS_SERVICE_DBUS,
 					       DBUS_PATH_DBUS,
 					       DBUS_INTERFACE_DBUS);
 
-	if (!dbus_g_proxy_call (bus_proxy, "RequestName", &error,
-		G_TYPE_STRING, GPM_DBUS_SERVICE,
-		G_TYPE_UINT, 0,
-		G_TYPE_INVALID,
-		G_TYPE_UINT, &request_name_result,
-		G_TYPE_INVALID)) {
-		g_warning ("Failed to acquire %s: %s", GPM_DBUS_SERVICE, error->message);
+	ret = dbus_g_proxy_call (bus_proxy, "RequestName", &error,
+				 G_TYPE_STRING, GPM_DBUS_SERVICE,
+				 G_TYPE_UINT, 0,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, &request_name_result,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("RequestName failed!");
 		return FALSE;
 	}
 
- 	if (request_name_result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
-		return FALSE;
-
 	/* free the bus_proxy */
 	g_object_unref (G_OBJECT (bus_proxy));
 
+	/* already running */
+ 	if (request_name_result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
+		return FALSE;
+	}
+
 	dbus_g_object_type_install_info (GPM_TYPE_MANAGER, &dbus_glib_gpm_manager_object_info);
 	dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH, object);
 
@@ -122,19 +131,19 @@ timed_exit_cb (GMainLoop *loop)
 int
 main (int argc, char *argv[])
 {
-	GMainLoop       *loop;
-	GnomeClient     *master;
+	GMainLoop *loop;
+	GnomeClient *master;
 	GnomeClientFlags flags;
 	DBusGConnection *system_connection;
 	DBusGConnection *session_connection;
-	gboolean	 verbose = FALSE;
-	gboolean	 no_daemon = FALSE;
-	gboolean	 timed_exit = FALSE;
-	GpmManager      *manager = NULL;
-	GError		*error = NULL;
-	GOptionContext  *context;
- 	GnomeProgram    *program;
-	char **debugoptions;
+	gboolean verbose = FALSE;
+	gboolean no_daemon = FALSE;
+	gboolean timed_exit = FALSE;
+	GpmManager *manager = NULL;
+	GError *error = NULL;
+	GOptionContext *context;
+ 	GnomeProgram *program;
+	char **debugoptions = NULL;
 	int i;
 
 	const GOptionEntry options[] = {
@@ -167,7 +176,6 @@ main (int argc, char *argv[])
 	master = gnome_master_client ();
 	flags = gnome_client_get_flags (master);
 
-
 	if (flags & GNOME_CLIENT_IS_CONNECTED) {
 		/* We'll disable this as users are getting constant crashes */
 		/* gnome_client_set_restart_style (master, GNOME_RESTART_IMMEDIATELY);*/
@@ -183,8 +191,15 @@ main (int argc, char *argv[])
 	gpm_debug_init (verbose);
 
 	/* Add all of the options specified on the --debug line */
-	for (i = 0; debugoptions[i]; i++) {
-		gpm_add_debug_option (debugoptions[i]);
+	if (debugoptions) {
+		for (i = 0; debugoptions[i]; i++) {
+			gpm_add_debug_option (debugoptions[i]);
+		}
+	}
+
+	/* we need to daemonize before we get a system connection to fix #366057 */
+	if (! no_daemon && daemon (0, 0)) {
+		gpm_critical_error ("Could not daemonize: %s", g_strerror (errno));
 	}
 
 	/* check dbus connections, exit if not valid */
@@ -216,10 +231,6 @@ main (int argc, char *argv[])
 		gpm_critical_error ("Cannot continue without stock icons");
 	}
 
-	if (! no_daemon && daemon (0, 0)) {
-		gpm_critical_error ("Could not daemonize: %s", g_strerror (errno));
-	}
-
 	manager = gpm_manager_new ();
 
 	if (!gpm_object_register (session_connection, G_OBJECT (manager))) {
@@ -227,8 +238,6 @@ main (int argc, char *argv[])
 		return 0;
 	}
 
-	dbus_g_object_type_install_info (GPM_TYPE_MANAGER, &dbus_glib_gpm_manager_object_info);
-
 	loop = g_main_loop_new (NULL, FALSE);
 
 	/* Only timeout and close the mainloop if we have specified it
