@@ -29,18 +29,20 @@
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <dbus/dbus-glib.h>
-#include <gconf/gconf-client.h>
 #include <math.h>
 #include <string.h>
 
+#include "gpm-tray-icon.h"
 #include "gpm-common.h"
 #include "gpm-prefs.h"
+#include "gpm-conf.h"
 #include "gpm-hal.h"
-#include "gpm-hal-cpufreq.h"
+#include "gpm-cpufreq.h"
 #include "gpm-prefs-core.h"
 #include "gpm-debug.h"
 #include "gpm-stock-icons.h"
 #include "gpm-screensaver.h"
+#include "gpm-powermanager.h"
 
 static void     gpm_prefs_class_init (GpmPrefsClass *klass);
 static void     gpm_prefs_init       (GpmPrefs      *prefs);
@@ -51,17 +53,17 @@ static void     gpm_prefs_finalize   (GObject	    *object);
 struct GpmPrefsPrivate
 {
 	GladeXML		*glade_xml;
-	GConfClient		*gconf_client;
 	gboolean		 has_batteries;
 	gboolean		 has_lcd;
 	gboolean		 has_ups;
 	gboolean		 has_button_lid;
-	gboolean		 has_cpufreq;
+	gboolean		 has_button_suspend;
 	gboolean		 can_suspend;
 	gboolean		 can_hibernate;
+	GpmConf			*conf;
 	GpmScreensaver		*screensaver;
-	GpmHalCpuFreq		*cpufreq;
-	GpmHalCpuFreqEnum	 cpufreq_types;
+	GpmCpuFreq		*cpufreq;
+	GpmCpuFreqEnum		 cpufreq_types;
 	GpmHal			*hal;
 };
 
@@ -75,15 +77,6 @@ static guint	     signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GpmPrefs, gpm_prefs, G_TYPE_OBJECT)
 
-static GConfEnumStringPair icon_policy_enum_map [] = {
-	{ GPM_ICON_POLICY_ALWAYS,	"always"   },
-	{ GPM_ICON_POLICY_PRESENT,	"present"  },
-	{ GPM_ICON_POLICY_CHARGE,	"charge"   },
-	{ GPM_ICON_POLICY_CRITICAL,	"critical" },
-	{ GPM_ICON_POLICY_NEVER,	"never"    },
-	{ 0, NULL }
-};
-
 /* The text that should appear in the action combo boxes */
 #define ACTION_INTERACTIVE_TEXT		_("Ask me")
 #define ACTION_SUSPEND_TEXT		_("Suspend")
@@ -100,12 +93,7 @@ static GConfEnumStringPair icon_policy_enum_map [] = {
 #define CPUFREQ_PERFORMANCE_TEXT	_("Maximum speed")
 #define CPUFREQ_NOTHING_TEXT 		_("Do nothing")
 
-#define	GPM_DBUS_SERVICE		"org.gnome.PowerManager"
-#define	GPM_DBUS_PATH			"/org/gnome/PowerManager"
-#define	GPM_DBUS_INTERFACE		"org.gnome.PowerManager"
-
-/* If sleep time in a slider is set to 61 it is considered as never sleep */
-const int NEVER_TIME_ON_SLIDER = 61;
+#define GPM_SLEEP_MAX_TIME		60
 
 /**
  * gpm_prefs_class_init:
@@ -162,9 +150,8 @@ gpm_dbus_method_bool (const gchar *method)
 	DBusGConnection *connection;
 	DBusGProxy *proxy;
 	GError *error;
-	gboolean value;
-
-	value = FALSE;
+	gboolean ret;
+	gboolean value = FALSE;
 	error = NULL;
 	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 	if (connection == NULL) {
@@ -180,22 +167,20 @@ gpm_dbus_method_bool (const gchar *method)
 					   GPM_DBUS_SERVICE,
 					   GPM_DBUS_PATH,
 					   GPM_DBUS_INTERFACE);
-	if (! dbus_g_proxy_call (proxy,
-				 method,
-				 &error,
+	ret = dbus_g_proxy_call (proxy, method, &error,
 				 G_TYPE_INVALID,
 				 G_TYPE_BOOLEAN, &value,
-				 G_TYPE_INVALID)) {
-		if (error) {
-			gpm_warning ("Couldn't connect to PowerManager %s",
-				     error->message);
-			g_error_free (error);
-		}
-		value = FALSE;
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("%s failed!", method);
+		return FALSE;
 	}
-
 	g_object_unref (proxy);
-
 	return value;
 }
 
@@ -224,11 +209,9 @@ gpm_prefs_icon_radio_cb (GtkWidget *widget,
 	gint policy;
 
 	policy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "policy"));
-	str = gconf_enum_to_string (icon_policy_enum_map, policy);
-	gpm_debug ("Changing %s to %s", GPM_PREF_ICON_POLICY, str);
-	gconf_client_set_string (prefs->priv->gconf_client,
-				 GPM_PREF_ICON_POLICY,
-				 str, NULL);
+	str = gpm_tray_icon_mode_to_string (policy);
+	gpm_debug ("Changing %s to %s", GPM_CONF_ICON_POLICY, str);
+	gpm_conf_set_string (prefs->priv->conf, GPM_CONF_ICON_POLICY, str);
 }
 
 /**
@@ -255,14 +238,7 @@ gpm_prefs_format_time_cb (GtkScale *scale,
 			  GpmPrefs *prefs)
 {
 	gchar *str;
-	if ((gint) value == NEVER_TIME_ON_SLIDER) {
-		str = g_strdup (_("Never"));
-	} else {
-		/* we auto-add the gss idle time to stop users getting
-		   confused. */
-		str = gpm_get_timestring (value * 60);
-	}
-
+	str = gpm_get_timestring (value * 60);
 	return str;
 }
 
@@ -281,23 +257,18 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
 
 	value = (int) gtk_range_get_value (range);
 
-	if (value == NEVER_TIME_ON_SLIDER) {
-		/* power manager interprets 0 as Never */
-		value = 0;
-	} else {
-		/* We take away the g-s idle time as the slider represents
-		 * global time but we only do our timeout from when g-s
-		 * declares the session idle */
-		gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
-		value -= gs_idle_time;
+	/* We take away the g-s idle time as the slider represents
+	 * global time but we only do our timeout from when g-s
+	 * declares the session idle */
+	gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	value -= gs_idle_time;
 
-		/* policy is in seconds, slider is in minutes */
-		value *= 60;
-	}
+	/* policy is in seconds, slider is in minutes */
+	value *= 60;
 
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "gconf_key");
+	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "conf_key");
 	gpm_debug ("Changing %s to %i", gpm_pref_key, value);
-	gconf_client_set_int (prefs->priv->gconf_client, gpm_pref_key, value, NULL);
+	gpm_conf_set_int (prefs->priv->conf, gpm_pref_key, value);
 }
 
 /**
@@ -314,31 +285,25 @@ gpm_prefs_setup_sleep_slider (GpmPrefs    *prefs,
 	GtkWidget *widget;
 	gint value;
 	gboolean is_writable;
+	guint gs_idle_time;
 
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
 	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_time_cb), prefs);
 
-	value = gconf_client_get_int (prefs->priv->gconf_client, gpm_pref_key, NULL);
-
-	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
-						    gpm_pref_key, NULL);
+	gpm_conf_get_int (prefs->priv->conf, gpm_pref_key, &value);
+	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
-	if (value == 0) {
-		value = NEVER_TIME_ON_SLIDER;
-	} else {
-		int gs_idle_time;
-		/* policy is in seconds, slider is in minutes */
-		value /= 60;
-		gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
-		value += gs_idle_time;
-	}
+	/* policy is in seconds, slider is in minutes */
+	value /= 60;
+	gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	value += gs_idle_time;
 
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
-	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
 
 	g_signal_connect (G_OBJECT (widget), "value-changed",
 			  G_CALLBACK (gpm_prefs_sleep_slider_changed_cb),
@@ -360,11 +325,11 @@ gpm_prefs_brightness_slider_changed_cb (GtkRange *range,
 	gchar *gpm_pref_key;
 
 	value = gtk_range_get_value (range);
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "gconf_key");
+	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "conf_key");
 
-	g_object_set_data (G_OBJECT (range), "gconf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (range), "conf_key", (gpointer) gpm_pref_key);
 	gpm_debug ("Changing %s to %i", gpm_pref_key, (int) value);
-	gconf_client_set_int (prefs->priv->gconf_client, gpm_pref_key, (gint) value, NULL);
+	gpm_conf_set_int (prefs->priv->conf, gpm_pref_key, (gint) value);
 }
 
 /**
@@ -388,15 +353,14 @@ gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
 	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_percentage_cb), NULL);
 
-	value = gconf_client_get_int (prefs->priv->gconf_client, gpm_pref_key, NULL);
-	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
-						    gpm_pref_key, NULL);
+	gpm_conf_get_int (prefs->priv->conf, gpm_pref_key, &value);
+	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
-	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
 
 	g_signal_connect (G_OBJECT (widget), "value-changed",
 			  G_CALLBACK (gpm_prefs_brightness_slider_changed_cb),
@@ -404,6 +368,26 @@ gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
 	return widget;
 }
 
+/**
+ * gpm_prefs_enable_widget:
+ **/
+static gboolean
+gpm_prefs_enable_widget (GpmPrefs    *prefs,
+		       const gchar *widgetname,
+		       gboolean     show)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, widgetname);
+	if (widget == NULL) {
+		gpm_warning ("widget '%s' not found", widgetname);
+		return FALSE;
+	}
+
+	gtk_widget_set_sensitive (widget, show);
+	return TRUE;
+}
+
 /**
  * gpm_prefs_action_combo_changed_cb:
  * @widget: The GtkWidget object
@@ -416,6 +400,7 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	gchar *value;
 	const gchar *action;
 	gchar *gpm_pref_key;
+	const gchar *widget_name;
 
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 
@@ -436,9 +421,16 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	}
 
 	g_free (value);
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "gconf_key");
+	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	gpm_debug ("Changing %s to %s", gpm_pref_key, action);
-	gconf_client_set_string (prefs->priv->gconf_client, gpm_pref_key, action, NULL);
+	gpm_conf_set_string (prefs->priv->conf, gpm_pref_key, action);
+
+	widget_name = gtk_widget_get_name (widget);
+	if (widget_name && strcmp (widget_name, "combobox_sleep_ac_type") == 0) {
+		gpm_prefs_enable_widget (prefs, "hbox_sleep_ac_custom", action != ACTION_NOTHING);
+	} else if (widget_name && strcmp (widget_name, "combobox_sleep_battery_type") == 0) {
+		gpm_prefs_enable_widget (prefs, "hbox_sleep_battery_custom", action != ACTION_NOTHING);
+	}
 }
 
 /**
@@ -463,13 +455,15 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
 
 	widget = glade_xml_get_widget (xml, widget_name);
 
-	value = gconf_client_get_string (prefs->priv->gconf_client,
-					 gpm_pref_key, NULL);
-	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
-						    gpm_pref_key, NULL);
+	gpm_conf_get_string (prefs->priv->conf, gpm_pref_key, &value);
+	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
+	g_signal_connect (G_OBJECT (widget), "changed",
+			  G_CALLBACK (gpm_prefs_action_combo_changed_cb), prefs);
+
 	if (value == NULL) {
 		gpm_warning ("invalid schema, please re-install");
 		value = g_strdup ("nothing");
@@ -505,7 +499,7 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
 						   ACTION_NOTHING_TEXT);
 			n_added++;
 		} else {
-			gpm_critical_error ("Unknown action read from gconf: %s",
+			gpm_critical_error ("Unknown action read from conf: %s",
 					    actions[i]);
 		}
 
@@ -513,10 +507,6 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
 			 gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added - 1);
 		i++;
 	}
-	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
-	g_signal_connect (G_OBJECT (widget), "changed",
-			  G_CALLBACK (gpm_prefs_action_combo_changed_cb),
-			  prefs);
 
 	g_free (value);
 }
@@ -553,9 +543,9 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
 		gtk_widget_set_sensitive (twidget, checked);
 	}
 
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "gconf_key");
+	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	gpm_debug ("Changing %s to %i", gpm_pref_key, checked);
-	gconf_client_set_bool (prefs->priv->gconf_client, gpm_pref_key, checked, NULL);
+	gpm_conf_set_bool (prefs->priv->conf, gpm_pref_key, checked);
 }
 
 /**
@@ -578,10 +568,10 @@ gpm_prefs_setup_checkbox (GpmPrefs    *prefs,
 
 	widget = glade_xml_get_widget (xml, widget_name);
 
-	checked = gconf_client_get_bool (prefs->priv->gconf_client, gpm_pref_key, NULL);
+	gpm_conf_get_bool (prefs->priv->conf, gpm_pref_key, &checked);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 
-	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_prefs_checkbox_lock_cb), prefs);
 
@@ -634,17 +624,17 @@ set_idle_hscale_stops (GpmPrefs    *prefs,
 {
 	GtkWidget *widget;
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
-	if (gs_idle_time + 1 > NEVER_TIME_ON_SLIDER) {
+	if (gs_idle_time + 1 > GPM_SLEEP_MAX_TIME) {
 		gpm_warning ("gnome-screensaver timeout is really big. "
 			     "Not sure what to do");
 		return;
 	}
-	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, NEVER_TIME_ON_SLIDER);
+	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, GPM_SLEEP_MAX_TIME);
 }
 
 /**
  * gs_delay_changed_cb:
- * @key: The gconf key
+ * @key: The conf key
  * @prefs: This prefs class instance
  **/
 static void
@@ -660,38 +650,27 @@ gs_delay_changed_cb (GpmScreensaver *screensaver,
 }
 
 /**
- * gconf_key_changed_cb:
+ * conf_key_changed_cb:
  *
- * We might have to do things when the gconf keys change; do them here.
+ * We might have to do things when the conf keys change; do them here.
  **/
 static void
-gconf_key_changed_cb (GConfClient *client,
-		      guint	   cnxn_id,
-		      GConfEntry  *entry,
-		      gpointer	   user_data)
+conf_key_changed_cb (GpmConf     *conf,
+		     const gchar *key,
+		     GpmPrefs    *prefs)
 {
-	GpmPrefs *prefs = GPM_PREFS (user_data);
 	gboolean  enabled;
 
-	gpm_debug ("Key changed %s", entry->key);
-
-	if (gconf_entry_get_value (entry) == NULL) {
-		return;
-	}
-
-	if (strcmp (entry->key, GPM_PREF_AC_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
-				  		 GPM_PREF_AC_LOWPOWER, NULL);
+	if (strcmp (key, GPM_CONF_AC_LOWPOWER) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_AC_LOWPOWER, &enabled);
 		gpm_debug ("need to enable checkbox");
 
-	} else if (strcmp (entry->key, GPM_PREF_UPS_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
-				  		 GPM_PREF_UPS_LOWPOWER, NULL);
+	} else if (strcmp (key, GPM_CONF_UPS_LOWPOWER) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UPS_LOWPOWER, &enabled);
 		gpm_debug ("need to enable checkbox");
 
-	} else if (strcmp (entry->key, GPM_PREF_BATTERY_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
-				  		 GPM_PREF_BATTERY_LOWPOWER, NULL);
+	} else if (strcmp (key, GPM_CONF_BATTERY_LOWPOWER) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_BATTERY_LOWPOWER, &enabled);
 		gpm_debug ("need to enable checkbox");
 	}
 }
@@ -709,9 +688,9 @@ gpm_prefs_processor_slider_changed_cb (GtkRange *range,
 	gchar *gpm_pref_key;
 
 	value = (gint) gtk_range_get_value (range);
-	gpm_pref_key = (gchar *) g_object_get_data (G_OBJECT (range), "gconf_key");
+	gpm_pref_key = (gchar *) g_object_get_data (G_OBJECT (range), "conf_key");
 	gpm_debug ("Changing %s to %i", gpm_pref_key, value);
-	gconf_client_set_int (prefs->priv->gconf_client, gpm_pref_key, value, NULL);
+	gpm_conf_set_int (prefs->priv->conf, gpm_pref_key, value);
 }
 
 /**
@@ -733,16 +712,14 @@ gpm_prefs_setup_processor_slider (GpmPrefs   *prefs,
 	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_percentage_cb), prefs);
 
-	value = gconf_client_get_int (prefs->priv->gconf_client, gpm_pref_key, NULL);
-
-	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
-						    gpm_pref_key, NULL);
+	gpm_conf_get_int (prefs->priv->conf, gpm_pref_key, &value);
+	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
-	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
 
 	g_signal_connect (G_OBJECT (widget), "value-changed",
 			  G_CALLBACK (gpm_prefs_processor_slider_changed_cb),
@@ -764,11 +741,10 @@ gpm_prefs_processor_combo_changed_cb (GtkWidget *widget,
 	const gchar *policy;
 	gchar *gpm_pref_key;
 	gboolean show_custom = FALSE;
-	GtkWidget *twidget;
 
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 	if (value == NULL) {
-		g_warning ("active text failed");
+		gpm_warning ("active text failed");
 		return;
 	}
 	if (strcmp (value, CPUFREQ_ONDEMAND_TEXT) == 0) {
@@ -790,25 +766,15 @@ gpm_prefs_processor_combo_changed_cb (GtkWidget *widget,
 
 	/* show other options */
 	if (strcmp (gtk_widget_get_name (widget), "combobox_processor_ac_profile") == 0) {
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"label_processor_ac_custom");
-		gtk_widget_set_sensitive (twidget, show_custom);
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"hscale_processor_ac_custom");
-		gtk_widget_set_sensitive (twidget, show_custom);
+		gpm_prefs_enable_widget (prefs, "hbox_processor_ac_custom", show_custom);
 	} else {
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"label_processor_battery_custom");
-		gtk_widget_set_sensitive (twidget, show_custom);
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"hscale_processor_battery_custom");
-		gtk_widget_set_sensitive (twidget, show_custom);
+		gpm_prefs_enable_widget (prefs, "hbox_processor_battery_custom", show_custom);
 	}
 
 	g_free (value);
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "gconf_key");
+	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	gpm_debug ("Changing %s to %s", gpm_pref_key, policy);
-	gconf_client_set_string (prefs->priv->gconf_client, gpm_pref_key, policy, NULL);
+	gpm_conf_set_string (prefs->priv->conf, gpm_pref_key, policy);
 }
 
 /**
@@ -822,20 +788,18 @@ static void
 gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 				 const gchar      *widget_name,
 				 const gchar      *gpm_pref_key,
-				 GpmHalCpuFreqEnum cpufreq_types)
+				 GpmCpuFreqEnum cpufreq_types)
 {
 	gchar *value;
 	guint n_added = 0;
 	gboolean has_option = FALSE;
 	gboolean is_writable;
 	GtkWidget *widget;
-	GpmHalCpuFreqEnum cpufreq_type;
+	GpmCpuFreqEnum cpufreq_type;
 
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
-	value = gconf_client_get_string (prefs->priv->gconf_client,
-					 gpm_pref_key, NULL);
-	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
-						    gpm_pref_key, NULL);
+	gpm_conf_get_string (prefs->priv->conf, gpm_pref_key, &value);
+	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
@@ -844,12 +808,12 @@ gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 		value = g_strdup ("nothing");
 	}
 
-	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
 	g_signal_connect (G_OBJECT (widget), "changed",
 			  G_CALLBACK (gpm_prefs_processor_combo_changed_cb),
 			  prefs);
 
-	cpufreq_type = gpm_hal_cpufreq_string_to_enum (value);
+	cpufreq_type = gpm_cpufreq_string_to_enum (value);
 
 	if (cpufreq_types & GPM_CPUFREQ_ONDEMAND) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
@@ -919,21 +883,28 @@ prefs_setup_sleep (GpmPrefs *prefs)
 					     NULL};
 
 	/* Sleep Type Combo Box */
-	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_general_type",
-				      GPM_PREF_SLEEP_TYPE,
+	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_ac_type",
+				      GPM_CONF_AC_SLEEP_TYPE,
 				      sleep_type_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_battery_type",
+				      GPM_CONF_BATTERY_SLEEP_TYPE,
+				      sleep_type_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_ups_type",
+				      GPM_CONF_BATTERY_SLEEP_TYPE, /* fixme */
+				      sleep_type_actions);
+
 	/* Sleep time until we sleep */
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ac_inactive",
-				      GPM_PREF_AC_SLEEP_COMPUTER);
+				      GPM_CONF_AC_SLEEP_COMPUTER);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_battery_inactive",
-				      GPM_PREF_BATTERY_SLEEP_COMPUTER);
-//FIXME
-//	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ups_inactive",
-//				      GPM_PREF_UPS_SLEEP_COMPUTER);
+				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ups_inactive",
+				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
 
 	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_sleep_battery_inactive", delay);
 	set_idle_hscale_stops (prefs, "hscale_sleep_ac_inactive", delay);
+	set_idle_hscale_stops (prefs, "hscale_sleep_battery_inactive", delay);
+	set_idle_hscale_stops (prefs, "hscale_sleep_ups_inactive", delay);
 
 	if (prefs->priv->has_batteries == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_sleep_battery");
@@ -954,22 +925,22 @@ prefs_setup_display (GpmPrefs *prefs)
 
 	/* Sleep time for display to blank */
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_display_ac_sleep",
-				      GPM_PREF_AC_SLEEP_DISPLAY);
+				      GPM_CONF_AC_SLEEP_DISPLAY);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_display_battery_sleep",
-				      GPM_PREF_BATTERY_SLEEP_DISPLAY);
+				      GPM_CONF_BATTERY_SLEEP_DISPLAY);
 	/* Display brightness */
 	gpm_prefs_setup_brightness_slider (prefs, "hscale_display_ac_brightness",
-					   GPM_PREF_AC_BRIGHTNESS);
+					   GPM_CONF_AC_BRIGHTNESS);
 	gpm_prefs_setup_brightness_slider (prefs, "hscale_display_battery_brightness",
-					   GPM_PREF_BATTERY_BRIGHTNESS);
+					   GPM_CONF_BATTERY_BRIGHTNESS);
 
 	/* set up the general checkboxes */
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_dim",
-				  GPM_PREF_DISPLAY_IDLE_DIM);
+				  GPM_CONF_DISPLAY_IDLE_DIM);
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_state_change",
-				  GPM_PREF_DISPLAY_STATE_CHANGE);
+				  GPM_CONF_DISPLAY_STATE_CHANGE);
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_ambient",
-				  GPM_PREF_DISPLAY_STATE_CHANGE);
+				  GPM_CONF_DISPLAY_STATE_CHANGE);
 	/* for now, hide */
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_ambient");
 	gtk_widget_hide_all (widget);
@@ -979,19 +950,22 @@ prefs_setup_display (GpmPrefs *prefs)
 	set_idle_hscale_stops (prefs, "hscale_display_ac_sleep", delay);
 
 	if (prefs->priv->has_lcd == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hscale_display_ac_brightness");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_display_ac_brightness");
 		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_display_ac_brightness");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_display_battery_brightness");
 		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hscale_display_battery_brightness");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_dim");
 		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_display_battery_brightness");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_ac_dim");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_battery_dim");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_state_change");
 		gtk_widget_hide_all (widget);
 	}
+
+	/* the box will be empty */
+	if (prefs->priv->has_lcd == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_display_general");
+		gtk_widget_hide_all (widget);
+	}
+
 	if (prefs->priv->has_batteries == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_display_battery");
 		gtk_widget_hide_all (widget);
@@ -1007,7 +981,7 @@ prefs_setup_processor (GpmPrefs *prefs)
 	gint	      page;
 
 	/* remove if we have no options to set */
-	if (prefs->priv->has_cpufreq == FALSE) {
+	if (prefs->priv->cpufreq == NULL) {
 		notebook = glade_xml_get_widget (prefs->priv->glade_xml, "notebook_preferences");
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_processor");
 		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
@@ -1016,14 +990,14 @@ prefs_setup_processor (GpmPrefs *prefs)
 	}
 
 	gpm_prefs_setup_processor_slider (prefs, "hscale_processor_battery_custom",
-					  GPM_PREF_BATTERY_CPUFREQ_VALUE);
+					  GPM_CONF_BATTERY_CPUFREQ_VALUE);
 	gpm_prefs_setup_processor_slider (prefs, "hscale_processor_ac_custom",
-					  GPM_PREF_AC_CPUFREQ_VALUE);
+					  GPM_CONF_AC_CPUFREQ_VALUE);
 
 	gpm_prefs_setup_processor_combo (prefs, "combobox_processor_ac_profile",
-					 GPM_PREF_AC_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+					 GPM_CONF_AC_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
 	gpm_prefs_setup_processor_combo (prefs, "combobox_processor_battery_profile",
-					 GPM_PREF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+					 GPM_CONF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
 }
 
 /** setup the actions page */
@@ -1031,62 +1005,76 @@ static void
 prefs_setup_actions (GpmPrefs *prefs)
 {
 	GtkWidget    *widget;
-	const gchar  *button_lid_actions[] = {ACTION_NOTHING,
-					      ACTION_BLANK,
-					      ACTION_SUSPEND,
-					      ACTION_HIBERNATE,
-					      NULL};
-	const gchar  *battery_critical_actions[] = {ACTION_NOTHING,
-						    ACTION_SUSPEND,
-						    ACTION_HIBERNATE,
-						    ACTION_SHUTDOWN,
-						    NULL};
-	const gchar  *battery_ups_actions[] = {ACTION_NOTHING,
-					       ACTION_HIBERNATE,
-					       ACTION_SHUTDOWN,
-					       NULL};
-	const gchar  *power_button_actions[] = {ACTION_INTERACTIVE,
-						ACTION_SUSPEND,
-						ACTION_HIBERNATE,
-						ACTION_SHUTDOWN,
-						NULL};
+	const gchar  *button_lid_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_BLANK,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 NULL};
+	const gchar  *suspend_button_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 NULL};
+	const gchar  *battery_critical_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 ACTION_SHUTDOWN,
+				 NULL};
+	const gchar  *battery_ups_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_HIBERNATE,
+				 ACTION_SHUTDOWN,
+				 NULL};
+	const gchar  *power_button_actions[] =
+				{ACTION_INTERACTIVE,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 ACTION_SHUTDOWN,
+				 NULL};
 	/* Power button action */
 	gpm_prefs_setup_action_combo (prefs, "combobox_actions_general_power",
-				      GPM_PREF_BUTTON_POWER,
+				      GPM_CONF_BUTTON_POWER,
 				      power_button_actions);
 
+	/* Suspend button action */
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_general_suspend",
+				      GPM_CONF_BUTTON_SUSPEND,
+				      suspend_button_actions);
+
 	/* Lid close actions */
 	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ac_lid",
-				      GPM_PREF_AC_BUTTON_LID,
+				      GPM_CONF_AC_BUTTON_LID,
 				      button_lid_actions);
 	gpm_prefs_setup_action_combo (prefs, "combobox_actions_battery_lid",
-				      GPM_PREF_BATTERY_BUTTON_LID,
+				      GPM_CONF_BATTERY_BUTTON_LID,
 				      button_lid_actions);
 
 	/* set up the LowPowerMode checkbox */
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_actions_battery_low_power",
-	  			  GPM_PREF_BATTERY_LOWPOWER);
+	  			  GPM_CONF_BATTERY_LOWPOWER);
 
 	/* battery critical actions */
 	gpm_prefs_setup_action_combo (prefs, "combobox_actions_battery_critical",
-				      GPM_PREF_BATTERY_CRITICAL,
+				      GPM_CONF_BATTERY_CRITICAL,
 				      battery_critical_actions);
 	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_critical",
-				      GPM_PREF_UPS_CRITICAL,
+				      GPM_CONF_UPS_CRITICAL,
+				      battery_ups_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_low",
+				      GPM_CONF_UPS_LOW,
 				      battery_ups_actions);
 
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_low",
-				      GPM_PREF_UPS_LOW,
-				      battery_ups_actions);
+	if (prefs->priv->has_button_suspend == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_actions_general_suspend");
+		gtk_widget_hide_all (widget);
+	}
 
 	if (prefs->priv->has_button_lid == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "combobox_actions_ac_lid");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_actions_battery_lid");
 		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "combobox_actions_battery_lid");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_actions_ac_lid");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_actions_battery_lid");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_actions_ac_lid");
 		gtk_widget_hide_all (widget);
 		/* remove whole box as there is nothing in it */
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_ac");
@@ -1115,10 +1103,8 @@ prefs_setup_notification (GpmPrefs *prefs)
 	GtkWidget   *radiobutton_icon_never;
 	gboolean     is_writable;
 
-	icon_policy_str = gconf_client_get_string (prefs->priv->gconf_client,
-						   GPM_PREF_ICON_POLICY, NULL);
-	icon_policy = GPM_ICON_POLICY_ALWAYS;
-	gconf_string_to_enum (icon_policy_enum_map, icon_policy_str, &icon_policy);
+	gpm_conf_get_string (prefs->priv->conf, GPM_CONF_ICON_POLICY, &icon_policy_str);
+	icon_policy = gpm_tray_icon_mode_from_string (icon_policy_str);
 	g_free (icon_policy_str);
 
 	radiobutton_icon_always = glade_xml_get_widget (prefs->priv->glade_xml,
@@ -1132,8 +1118,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	radiobutton_icon_never = glade_xml_get_widget (prefs->priv->glade_xml,
 							"radiobutton_notification_never");
 
-	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
-						    GPM_PREF_ICON_POLICY, NULL);
+	gpm_conf_is_writable (prefs->priv->conf, GPM_CONF_ICON_POLICY, &is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_always, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_present, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_charge, is_writable);
@@ -1162,7 +1147,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	g_object_set_data (G_OBJECT (radiobutton_icon_never), "policy",
 			   GINT_TO_POINTER (GPM_ICON_POLICY_NEVER));
 
-	/* only connect the callbacks after we set the value, else the gconf
+	/* only connect the callbacks after we set the value, else the conf
 	   keys gets written to (for a split second), and the icon flickers. */
 	g_signal_connect (radiobutton_icon_always, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
@@ -1177,13 +1162,17 @@ prefs_setup_notification (GpmPrefs *prefs)
 
 	/* set up the sound checkbox */
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_notification_sound",
-	  			  GPM_PREF_ENABLE_BEEPING);
+	  			  GPM_CONF_ENABLE_BEEPING);
 
 	if (prefs->priv->has_batteries == FALSE) {
 		/* Hide battery radio options if we have no batteries */
 		gtk_widget_hide_all (radiobutton_icon_charge);
 		gtk_widget_hide_all (radiobutton_icon_critical);
 	}
+	if (prefs->priv->has_batteries == FALSE && prefs->priv->has_ups == FALSE) {
+		/* Hide battery present option if no ups or primary */
+		gtk_widget_hide_all (radiobutton_icon_present);
+	}
 }
 
 /**
@@ -1199,20 +1188,15 @@ gpm_prefs_init (GpmPrefs *prefs)
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
 	prefs->priv->hal = gpm_hal_new ();
-	prefs->priv->cpufreq = gpm_hal_cpufreq_new ();
+	prefs->priv->cpufreq = gpm_cpufreq_new ();
 
 	prefs->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (prefs->priv->screensaver, "gs-delay-changed",
 			  G_CALLBACK (gs_delay_changed_cb), prefs);
 
-	prefs->priv->gconf_client = gconf_client_get_default ();
-
-	gconf_client_notify_add (prefs->priv->gconf_client,
-				 GPM_PREF_DIR,
-				 gconf_key_changed_cb,
-				 prefs,
-				 NULL,
-				 NULL);
+	prefs->priv->conf = gpm_conf_new ();
+	g_signal_connect (prefs->priv->conf, "value-changed",
+			  G_CALLBACK (conf_key_changed_cb), prefs);
 
 	prefs->priv->has_lcd = gpm_hal_num_devices_of_capability (prefs->priv->hal, "laptop_panel") > 0;
 	prefs->priv->has_batteries = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
@@ -1224,24 +1208,25 @@ gpm_prefs_init (GpmPrefs *prefs)
 	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
 							"button.type",
 							"lid") > 0;
+	prefs->priv->has_button_suspend = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
+							"button.type",
+							"suspend") > 0;
 
 	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
 	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
 
 	/* only enable cpufreq stuff if we have the hardware */
-	prefs->priv->has_cpufreq = gpm_hal_cpufreq_has_hardware (prefs->priv->cpufreq);
-	gboolean ret = FALSE;
-	if (prefs->priv->has_cpufreq) {
-		ret = gpm_hal_cpufreq_get_governors (prefs->priv->cpufreq,
-							   &prefs->priv->cpufreq_types);
-	}
-	/* method failed, can't assume we have cpu frequency scaling */
-	if (ret == FALSE) {
+	if (prefs->priv->cpufreq) {
+		gpm_cpufreq_get_governors (prefs->priv->cpufreq,
+					       &prefs->priv->cpufreq_types);
+	} else {
 		prefs->priv->cpufreq_types = GPM_CPUFREQ_NOTHING;
-		prefs->priv->has_cpufreq = FALSE;
 	}
 
 	prefs->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
+	if (prefs->priv->glade_xml == NULL) {
+		g_error ("Cannot find 'gpm-prefs.glade'");
+	}
 
 	main_window = glade_xml_get_widget (prefs->priv->glade_xml, "window_preferences");
 
@@ -1294,11 +1279,16 @@ gpm_prefs_finalize (GObject *object)
 	prefs = GPM_PREFS (object);
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
-	g_object_unref (prefs->priv->gconf_client);
-	g_object_unref (prefs->priv->screensaver);
-	g_object_unref (prefs->priv->cpufreq);
-	g_object_unref (prefs->priv->hal);
-	/* FIXME: we should unref some stuff */
+	g_object_unref (prefs->priv->conf);
+	if (prefs->priv->screensaver) {
+		g_object_unref (prefs->priv->screensaver);
+	}
+	if (prefs->priv->cpufreq) {
+		g_object_unref (prefs->priv->cpufreq);
+	}
+	if (prefs->priv->hal) {
+		g_object_unref (prefs->priv->hal);
+	}
 
 	G_OBJECT_CLASS (gpm_prefs_parent_class)->finalize (object);
 }
